/* ----------------------------------------------------- */
/* Digitizer Check (digichek.c)                          */
/* Dave MacLeod <davmac@netcomuk.co.uk>                  */
/*                                                       */
/* This app show where the Pilot "thinks" the stylus is  */
/* touching the screen.                                  */
/* ----------------------------------------------------- */
/* v0.1     25/11/97    Creation                         */
/*                                                       */
/* ----------------------------------------------------- */

#pragma pack(2)
#include <System/SysAll.h>
#include <UI/UIAll.h>

#define MainForm     1000

#define HelpText     1000

#define FormAbout    1050
#define AboutDone    1051

#define menuAbout    3001
#define menuHelp     3005

#define hairsize     12

/* ----------------------------------------------------- */
/* Global Variables                                      */
/* ----------------------------------------------------- */

SWord XPos,
      YPos;

/* ----------------------------------------------------- */
/* GetObjPtr                                             */
/*   Return the pointer to a form object                 */
/* Parms                                                 */
/*   objectID - The ID of the object, specified in the   */
/*              .rcp file.                               */
/* Returns                                               */
/*   A pointer to the specified object                   */
/* ----------------------------------------------------- */

static VoidPtr GetObjPtr(Int objectID)
{
  FormPtr frm;

  frm = FrmGetActiveForm();
  return(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, objectID)));
}

/* ----------------------------------------------------- */
/* EraseOldCross                                         */
/*   Clear away the old cross when the stylus is moved   */
/*   or lifted.                                          */
/* Parms                                                 */
/*   None                                                */
/* Returns                                               */
/*   Nowt                                                */
/* ----------------------------------------------------- */

static void EraseOldCross(void)
{
  RectangleType rect;

  rect.topLeft.x=XPos-hairsize-1;
  rect.topLeft.y=YPos-hairsize-1;
  rect.extent.x=(hairsize*2)+2;
  rect.extent.y=(hairsize*2)+2;

  WinEraseRectangle(&rect,0);
}

/* ----------------------------------------------------- */
/* DrawNewCross                                          */
/*   Draw a new cross after the stylus has moved or put  */
/*   down.                                               */
/* Parms                                                 */
/*   None                                                */
/* Returns                                               */
/*   Nowt                                                */
/* ----------------------------------------------------- */

static void DrawNewCross(void)
{
  static char PosT[8],
              XPosT[4],
              YPosT[4];
  RectangleType rect;

  rect.topLeft.x=10;
  rect.topLeft.y=20;
  rect.extent.x=40;
  rect.extent.y=140;

  WinEraseRectangle(&rect,0);

  StrIToA(XPosT,XPos);
  StrIToA(YPosT,YPos);
  StrCopy(PosT,XPosT);
  StrCat(PosT,",");
  StrCat(PosT,YPosT);
  if (YPos<80)
  {
    WinDrawChars(PosT,StrLen(PosT),10,140);
  }
  else
  {
    WinDrawChars(PosT,StrLen(PosT),10,20);
  }
  WinDrawLine(XPos-hairsize,YPos,XPos+hairsize,YPos);
  WinDrawLine(XPos,YPos-hairsize,XPos,YPos+hairsize);

}

/* ----------------------------------------------------- */
/* AboutViewHandleEvent                                  */
/*   Handle something happening on the about form        */
/* Parms                                                 */
/*   event - Pointer to an event type structure          */
/* Returns                                               */
/*   True - if the event has been dealt with             */
/* ----------------------------------------------------- */

static Boolean AboutViewHandleEvent(EventPtr event)
{
  Boolean handled = false;
  FormType frm;

  switch (event->eType)
  {
    case frmOpenEvent:
      FrmDrawForm(FrmGetActiveForm());
      handled = true;
      break;
    case ctlEnterEvent:
      if (event->data.popSelect.controlID==AboutDone)
      {
        FrmGotoForm(MainForm);
        handled = true;
        break;
      }
  }
  return(handled);
}

/* ----------------------------------------------------- */
/* MainViewHandleEvent                                   */
/*   This function loads a form and set the event        */
/*   handler for the form.                               */
/* Parms                                                 */
/*   event - Pointer to an event type structure          */
/* Returns                                               */
/*   True - if the event has been dealt with             */
/* ----------------------------------------------------- */

static Boolean MainViewHandleEvent(EventPtr event)
{
  Boolean handled = false;
  FormType frm;
  RectangleType rect;

  switch (event->eType)
  {
    case frmOpenEvent:
      FrmDrawForm(FrmGetActiveForm());
      break;
    case penDownEvent:
      XPos = event->screenX;
      YPos = event->screenY;
      DrawNewCross();
      handled = false;
      break;
    case penMoveEvent:
      EraseOldCross();
      XPos = event->screenX;
      YPos = event->screenY;
      DrawNewCross();
      handled = false;
      break;
    case penUpEvent:
      EraseOldCross();
      FrmGotoForm(MainForm);
      handled = false;
      break;
    case menuEvent:
      MenuEraseStatus(0);
      switch (event->data.menu.itemID)
      {
        case menuHelp:
          FrmHelp(HelpText);
          handled = true;
          break;
        case menuAbout:
          FrmPopupForm(FormAbout);
          handled = true;
          break;
      }
      handled = true;
      break;
  }
  return(handled);
}

/* ----------------------------------------------------- */
/* ApplicationHandleEvent                                */
/*   This function loads a form and set the event        */
/*   handler for the form.                               */
/* Parms                                                 */
/*   event - Pointer to an event type structure          */
/* Returns                                               */
/*   True - if the event has been dealt with             */
/* ----------------------------------------------------- */

static Boolean ApplicationHandleEvent(EventPtr event)
{
  FormPtr frm;
  Int formId;
  Boolean handled = false;

  if (event->eType == frmLoadEvent)
  {
    formId = event->data.frmLoad.formID;
    frm = FrmInitForm(formId);
    FrmSetActiveForm(frm);

    switch (formId)
    {
      case MainForm:
        FrmSetEventHandler(frm, MainViewHandleEvent);
        break;
      case FormAbout:
        FrmSetEventHandler(frm, AboutViewHandleEvent);
        break;
    }
    handled = true;
  }
  return handled;
}

/* ----------------------------------------------------- */
/* InitApplication                                       */
/*   Get everything ready.                               */
/* Parms                                                 */
/*   None                                                */
/* Returns                                               */
/*   Nowt                                                */
/* ----------------------------------------------------- */

static void InitApplication(void)
{
  FrmGotoForm(MainForm);
  XPos = 50;
  YPos = 50;
}

/* ----------------------------------------------------- */
/* StopApplication                                       */
/*   Like my Mum used to say "Tidy up after yourself".   */
/* Parms                                                 */
/*   None                                                */
/* Returns                                               */
/*   Nowt                                                */
/* ----------------------------------------------------- */

static void StopApplication(void)
{
  FrmCloseAllForms ();
}

/* ----------------------------------------------------- */
/* EventLoop                                             */
/*   This function gets events from the Event Manager,   */
/*   passes them to the system, menu and application     */
/*   event handlers before passing them onto our own     */
/*   processing.                                         */
/* Parms                                                 */
/*   None                                                */
/* Returns                                               */
/*   Nowt                                                */
/* ----------------------------------------------------- */

static void EventLoop(void)
{
  EventType event;
  Word error;
  do
  {
    EvtGetEvent(&event, evtWaitForever);
    if (! SysHandleEvent(&event))
      if (! MenuHandleEvent(NULL, &event, &error))
        if (!ApplicationHandleEvent(&event))
          FrmDispatchEvent(&event);
  }
  while (event.eType != appStopEvent);
}

/* ----------------------------------------------------- */
/* PilotMain                                             */
/*   Equivalent of the normal C Main(), it starts the    */
/*   execution of the application.                       */
/* Parms                                                 */
/*   cmd - command specifying how to launch the app      */
/*   cmdPBP - parameter block for the command            */
/*   launchFlags - flags used to config the launch       */
/* Returns                                               */
/*   An applicable error code                            */
/* ----------------------------------------------------- */

DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
  Int err;

  if (cmd == sysAppLaunchCmdNormalLaunch)
  {
    InitApplication();
    EventLoop();
    StopApplication();
  }
  return 0;
}
